__import__("pkg_resources").declare_namespace(__name__)

import gc
import nose
from infi.memuse import get_rss, PotentialMemoryLeakError, DEFAULT_ALLOWED_LEFTOVER

class NosePlugin(nose.plugins.Plugin):
    name = 'infi-memuse'

    def __init__(self, *args, **kwargs):
        super(NosePlugin, self).__init__(*args, **kwargs)
        self.allowed_leftover = DEFAULT_ALLOWED_LEFTOVER

    def help(self):
        return "Checks tests for memory leaks"

    def configure(self, options, conf):
        nose.plugins.Plugin.configure(self, options, conf)
        if options.memuseLeftover:
            self.allowed_leftover = options.memuseLeftover

    def options(self, parser, env):
        nose.plugins.Plugin.options(self, parser, env)
        parser.add_option('--memuse-leftover', action='store', dest='memuseLeftover', 
                          default=DEFAULT_ALLOWED_LEFTOVER,
                          help=('Size in bytes of allowed memory footprint "leftover" after the test ends (default=%d)'
                                % DEFAULT_ALLOWED_LEFTOVER))

    def startTest(self, test):
        gc.disable()
        gc.collect()
        self.rss_baseline = get_rss()
        gc.enable()

    def stopTest(self, test):
        gc.disable()
        gc.collect()
        rss_current = get_rss()
        rss_delta = rss_current - self.rss_baseline
        gc.enable()

        if rss_delta > self.allowed_leftover:
            raise PotentialMemoryLeakError("test [%s]" % test, self.rss_baseline, rss_current, self.allowed_leftover)

    def prepareTestResult(self, result):
        # Monkey-patch the result object to convert errors created by us to failures.
        prev = result.addError
        
        def monkey_patched_addError(test, err):
            if isinstance(err[1], PotentialMemoryLeakError):
                result.addFailure(test, err)
            else:
                prev(test, err)

        result.addError = monkey_patched_addError



__version__ = "0.0.5"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.0.5)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = 'd339b2ac7935c87f79ed3afe654d84014898575d'
__git_head_subject__ = 'buildout.cfg: changed isolated python version to v2.7.5.5'
__git_head_message__ = ''
__git_dirty_diff__ = ''


__import__("pkg_resources").declare_namespace(__name__)

from psutil import Process
from os import getpid
import inspect
import gc

from infi.pyutils.decorators import wraps

DEFAULT_ALLOWED_LEFTOVER = 512 * 1024 * 1024

class PotentialMemoryLeakError(AssertionError):
    __MESSAGE__ =  \
        "\nPotential memory leak detected in {0}:\n" \
        "Baseline RSS was {1} bytes before test started, " \
        "but growed to be {2} bytes after test ended (delta={3} bytes).\n" \
        "Allowed memory leftover was {4} bytes - consider increasing it if there is no leak."

    def __init__(self, where, baseline_rss, current_rss, rss_leftover):
        super(PotentialMemoryLeakError, self).__init__(self.__MESSAGE__.format(where,
                                                                               baseline_rss, current_rss, 
                                                                               current_rss - baseline_rss,
                                                                               rss_leftover))

def get_rss():
    return get_self_rss() + get_children_rss()

def get_self_rss():
    pid = getpid()
    return Process(pid).get_memory_info().rss

def get_children_rss():
    pid = getpid()
    return sum([ child.get_memory_info().rss for child in Process(pid).get_children() ], 0)

def verify_rss(rss_leftover=DEFAULT_ALLOWED_LEFTOVER, ignore_methods=[]):
    def _decorator(func_or_class):
        if inspect.isclass(func_or_class):
            return __verify_rss_wrap_class(func_or_class, ignore_methods, rss_leftover)
        else:
            return __verify_rss_wrap_func(func_or_class, rss_leftover)
    return _decorator

def __verify_rss_wrap_class(cls, ignore_methods, rss_leftover):
    for attr_name in dir(cls):
        if not attr_name.startswith('__') and attr_name not in ignore_methods:
            attr = getattr(cls, attr_name)
            if inspect.isfunction(attr) or inspect.ismethod(attr):
                setattr(cls, attr_name, __verify_rss_wrap_func(attr, rss_leftover))
    return cls

def __verify_rss_wrap_func(func, rss_leftover):
    @wraps(func)
    def new_func(*args, **kwargs):
        gc.disable()
        gc.collect()
        rss_baseline = get_rss()
        gc.enable()
        try:
            result = func(*args, **kwargs)
        except:
            # Don't shadow the original exception if happened - we don't test for leak in this case.
            raise
        else:
            gc.disable()
            gc.collect()
            rss_current = get_rss()
            gc.enable()
            if (rss_current - rss_baseline) > rss_leftover:
                try:
                    where = 'file %s, function %s' % (inspect.getfile(func), func)
                except:
                    where = 'function %s' % (func,)
                raise PotentialMemoryLeakError(where, rss_baseline, rss_current, rss_leftover)
            
        return result
    return new_func




SETUP_INFO = dict(
    name = 'infi.memuse',
    version = '0.0.5',
    author = 'Guy Rozendorn',
    author_email = 'guy@rzn.co.il',

    url = 'http://www.infinidat.com',
    license = 'PSF',
    description = """short description here""",
    long_description = """long description here""",

    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Python Software Foundation License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

    install_requires = ['distribute', 'infi.pyutils', 'psutil>=1.2'],
    namespace_packages = [],

    package_dir = {'': 'src'},
    package_data = {'': []},
    include_package_data = True,
    zip_safe = False,

    entry_points = {
        'console_scripts': [],
        'gui_scripts': [],
        'nose.plugins': ['infi.memuse = infi.memuse.nose:NosePlugin']
        },
    )

def setup():
    from setuptools import setup as _setup
    from setuptools import find_packages
    SETUP_INFO['packages'] = find_packages('src')
    _setup(**SETUP_INFO)

if __name__ == '__main__':
    setup()

